//-----------------------------------------------------------------------------
// Copyright (c) 2004 Soft Gold Software Corporation.  All Rights Reserved.
//-----------------------------------------------------------------------------

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

import dxfExporter.*;

/**
 *  Frame class for testing a export to DXF.
 */
public class FrmDXFExporter extends JFrame {
  private static DXFExport Xprt;
  private String sgStr;
  private static final String AppName = new String("DXFExporter version demo");
  private static final String FilePath = new String("DEMOdxfExporter.dxf");
  private static final String strPixel = "Pixel";
  private static final String strLine = "Line";
  private static final String strPolyline = "Polyline";
  private static final String strRectangle = "Rectangle";
  private static final String strCircle = "Circle (+small hatch)";
  private static final String strEllipse = "Ellipse (+small hatch)";
  private static final String strArc = "Arc";
  private static final String strPolygon = "Polygon (hatch)";
  private static final String strBezier = "Bezier";
  private static final String str3DFace = "3DFace";
  private static final String strAll = "All figures";
  private static final String strSaveToDXFFile = "Save to DXF file";
  private static final String strRedraw = "Redraw";
  private static final Color sgTextColor = new Color(10, 155, 150);

  private JPanel contentPane;
  private JPanel jpnlTools;
  private JPanel jpnlDraw;
  private Graphics grphCanvas;
  private BorderLayout brdrLayout = new BorderLayout();
  private JButton jbtnSave = new JButton();
  private JComboBox jcbFigures = new JComboBox();
  Checkbox cbACAD2000 = new Checkbox();

  /*private class DemoDialog extends Dialog implements ActionListener {
    DemoDialog(Frame parent, String title, boolean modal) {
      super(parent, title, modal);
      setLayout(new FlowLayout());
      setSize(300, 200);
      Button btn;
      add(btn = new Button("Ok"));
      btn.addActionListener(this);
    }
    public void actionPerformed(ActionEvent ae) {
      dispose();
    }
  }*/

  //Construct the frame
  public FrmDXFExporter() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  //Component initialization
  private void jbInit() throws Exception  {
    Xprt = null;
    jpnlTools = new JPanel();
    jpnlDraw = new JPanel();
    grphCanvas = jpnlDraw.getGraphics();
    this.setResizable(false);
    this.setTitle(AppName);
    //setIconImage(Toolkit.getDefaultToolkit().createImage(FrmDXFExporter.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    jpnlTools.setLayout(new BorderLayout());
    jbtnSave.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jbtnSave_actionPerformed(e);
      }
    });
    jcbFigures.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jcbFigures_actionPerformed(e);
      }
    });
    jpnlDraw.setBorder(BorderFactory.createLoweredBevelBorder());
//    cbACAD2000.setActionCommand("jcbACAD2000");
    cbACAD2000.setLabel("AutoCAD 2000");
    jpnlTools.add(jbtnSave, BorderLayout.EAST);
    jpnlTools.add(jcbFigures, BorderLayout.WEST);
    jpnlTools.add(cbACAD2000, BorderLayout.CENTER);
    jcbFigures.addItem(strPixel);
    jcbFigures.addItem(strLine);
    jcbFigures.addItem(strPolyline);
    jcbFigures.addItem(strRectangle);
    jcbFigures.addItem(strCircle);
    jcbFigures.addItem(strEllipse);
    jcbFigures.addItem(strArc);
    jcbFigures.addItem(strPolygon);
    jcbFigures.addItem(strBezier);
    jcbFigures.addItem(str3DFace);
    jcbFigures.addItem(strAll);
    contentPane.setLayout(brdrLayout);
    contentPane.add(jpnlDraw, BorderLayout.CENTER);
    contentPane.add(jpnlTools, BorderLayout.SOUTH);
    setSize(new Dimension(520, 330));
    jbtnSave.setText(strSaveToDXFFile);
    jcbFigures.setSelectedItem(strAll);
    sgStr = (String) jcbFigures.getSelectedItem();
  }
  //
  // Overridden so we can exit when window is closed
  //
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }
  void jcbFigures_actionPerformed(ActionEvent e) {
    super.processEvent(e);
    sgStr = (String) jcbFigures.getSelectedItem();
    this.repaint();
  }
  void jbtnSave_actionPerformed(ActionEvent e) {
    super.processEvent(e);
    if (jbtnSave.getText().compareTo(strSaveToDXFFile) == 0 ) {
      Xprt = new DXFExport();
      if (cbACAD2000.getState())
        Xprt.AutoCADVer = Constants.DXFVERSION_R2000;
      Graphics prevDraw = grphCanvas;
      grphCanvas = null;
      DrawFigures();
      grphCanvas = prevDraw;
      try {
        Xprt.saveToFile(FilePath);
        sgStr = "DXF file \"" + FilePath + "\" successfully created...";
      }
      catch(Exception excpt) {
        sgStr = "ERROR: '"+excpt.getMessage()+"'. Cannot save to file...";
      }
      finally {
        Xprt.finalize();
        Xprt = null;
        jbtnSave.setText(strRedraw);
        jcbFigures.setEnabled(false);
        this.repaint();
      }
    }
    else {
      jbtnSave.setText(strSaveToDXFFile);
      jcbFigures.setEnabled(true);
      sgStr = (String) jcbFigures.getSelectedItem();
      this.repaint();
    }
  }
  /**
   * Paint Handler.
  */
  public void paint(Graphics g)
  {
    super.paint(g);
    grphCanvas = jpnlDraw.getGraphics();
    DrawFigures();
  }
  /**
   *  Draw figures.
   */
  private void DrawFigures() {
    if (sgStr.compareTo(strAll) == 0) {
      sgDrawPixel();
      sgDrawLine();
      sgDrawPolyline();
      sgDrawRectangle();
      sgDrawCircle();
      sgDrawEllipse();
      sgDrawArc();
      sgDrawPolygon();
      sgDrawBezier();
      sgDraw3DFace();
    }
    else if (sgStr.compareTo(strPixel) == 0)
      sgDrawPixel();
    else if (sgStr.compareTo(strLine) == 0)
      sgDrawLine();
    else if (sgStr.compareTo(strPolyline) == 0)
      sgDrawPolyline();
    else if (sgStr.compareTo(strRectangle) == 0)
      sgDrawRectangle();
    else if (sgStr.compareTo(strCircle) == 0)
      sgDrawCircle();
    else if (sgStr.compareTo(strEllipse) == 0)
      sgDrawEllipse();
    else if (sgStr.compareTo(strArc) == 0)
      sgDrawArc();
    else if (sgStr.compareTo(strPolygon) == 0)
      sgDrawPolygon();
    else if (sgStr.compareTo(strBezier) == 0)
      sgDrawBezier();
    else if (sgStr.compareTo(str3DFace) == 0)
      sgDraw3DFace();
    else
      sgDrawText();
  }
  /**
   * Draw a text.
   */
  public void sgDrawText() {
    grphCanvas.setColor(sgTextColor);
    grphCanvas.drawString(sgStr, 10, jpnlDraw.getHeight()/2);
  }
  /**
   * Draw a pixel.
  */
  public void sgDrawPixel() {
    int x=25, y=200;
    Color cl = Color.yellow;
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strPixel, x-10, y-3);
      grphCanvas.drawLine(x, y, x, y);
    }
    if (Xprt!= null) {
      DXFLayer Layer = new DXFLayer(strPixel);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = Constants.convertColorRGBToDXF(cl);
      Dt.Point = new DXFPoint(x, -y, 0);
      Xprt.addPixel(Dt);
      Dt.Point.setTo(x - 10, -(y - 3), 0);
      Dt.Point1 = new DXFPoint(Dt.Point.X, Dt.Point.Y, 0);
      Dt.FHeight = 10;
      Dt.HAlign = 1;
      Dt.Text = new String(strPixel);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a line.
   */
  public void sgDrawLine() {
    int x1=10, y1=180, x2=20, y2=40;
    Color cl = Color.blue;
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strLine, x1, y2-20);
      grphCanvas.drawLine(x1, y1, x2, y2);
    }
    if (Xprt!= null) {
      DXFLayer Layer = new DXFLayer(strLine);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Point = new DXFPoint(x1, -y1, 0);
      Dt.Point1 = new DXFPoint(x2, -y2, 0);
      Xprt.addLine(Dt);
      Dt.Point.setTo(x1, -(y2-20), 0);
      Dt.FHeight = 10;
      Dt.Text = new String(strLine);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a polyline.
   */
  public void sgDrawPolyline() {
    int arrX[]= {25,50,30,100,120}, arrY[]={90,60,130,50,110};
    Color cl = new Color(0,255,0);
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strPolyline, 44, 56);
      grphCanvas.drawPolyline(arrX, arrY, 5);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strPolyline);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Count = arrX.length;
      Dt.Points = new ArrayList(Dt.Count);
      for (int i = 0; i< Dt.Count; ++i)
    	  Dt.Points.add(new DXFPoint(arrX[i], -(arrY[i]), 0));
      Xprt.addPolyline(Dt);
      Dt.Point = new DXFPoint(44, -56, 0);
      Dt.FHeight = 6;
      Dt.Text = new String(strPolyline);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /*
   * Draw a rectangle.
   */
  public void sgDrawRectangle() {
    int x1=50, y1=120, x2=130, y2=180;
    Color cl = Color.black;
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strRectangle, x1+10, y1+16);
      grphCanvas.drawRect(x1, y1, x2-x1, y2-y1);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strRectangle);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Point = new DXFPoint(x1, -y1, 0);
      Dt.Point1 = new DXFPoint(x2, -y2, 0);
      Xprt.addRectangle(Dt);
      Dt.Point.setTo(x1+10, -(y1+16), 0);
      Dt.FHeight = 6;
      Dt.Text = new String(strRectangle);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a circle.
   */
  public void sgDrawCircle() {
    int x=110, y=50, radius=15;
    Color cl = new Color(0, 0, 128);
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strCircle, x-40, y-2);
      grphCanvas.drawOval(x, y, 2*radius, 2*radius);
      grphCanvas.fillOval(x+radius/2, y+radius/2, radius, radius);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strCircle);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Point = new DXFPoint(x+radius, -(y+radius), 0);
      Dt.Radius = radius;
      Xprt.addCircle(Dt);
      Dt.Radius = Dt.Radius / 2;
      Dt.SelfType = HatchBoundaryType.hbtCircle;
      Dt.StartAngle = 0;
      Dt.EndAngle = 360;
      Xprt.addHatch(Dt);
      Dt.FHeight = 4;
      Dt.Point.setTo(x, -(y-2), 0);
      Dt.Text = new String(strCircle);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a ellipse.
   */
  public void sgDrawEllipse() {
    int x=180, y=120, xradius=30, yradius=10;
    Color cl = Color.magenta;
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strEllipse, x-xradius, y-yradius-2);
      grphCanvas.drawOval(x-xradius, y-yradius, 2*xradius, 2*yradius);
      grphCanvas.fillOval(x-xradius/2, y-yradius/2, xradius, yradius);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strEllipse);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Point = new DXFPoint(x, -y, 0);
      Dt.Point1 = new DXFPoint(xradius, 0, 0);
      Dt.Radius = ((xradius >= yradius) ? ((float)yradius/xradius) : ((float)xradius/yradius));
      Dt.StartAngle = 0;
      Dt.EndAngle = 360;
      Xprt.addEllipse(Dt);
      Dt.Point1 = new DXFPoint(xradius/2, 0, 0);
      Dt.Radius = Dt.Radius / 2;
      Dt.SelfType = HatchBoundaryType.hbtEllipse;
      Xprt.addHatch(Dt);
      Dt.FHeight = 4;
      Dt.Point.setTo(x-xradius, -(y-yradius-2), 0);
      Dt.Text = new String(strEllipse);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a arc.
   */
  public void sgDrawArc() {
    int x=180, y=180, radius=30;
    int stAng=30, arAng=260;
    Color cl = Color.gray;
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strArc, x-3, y);
      grphCanvas.drawArc(x-radius, y-radius, 2*radius, 2*radius, stAng, arAng);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strArc);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Point = new DXFPoint(x, -y, 0);
      Dt.StartAngle = stAng;
      Dt.EndAngle = arAng+stAng;
      Dt.Radius = radius;
      Xprt.addArc(Dt);
      Dt.FHeight = 12;
      Dt.Point.setTo(x-3, -y, 0);
      Dt.Text = new String(strArc);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a hatch.
   */
  public void sgDrawPolygon() {
    Color cl = Color.cyan;
    int arrX[]= {160, 168, 150, 172, 180, 188, 210, 192, 200, 180},
        arrY[]= { 95,  76,  65,  65,  45,  65,  65,  76,  95,  83};
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strPolygon, arrX[0], arrY[4]-2);
      grphCanvas.fillPolygon(arrX, arrY, arrX.length);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strPolygon);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Count = 1;
      Dt.Style = HatchStyle.hsSolid;// default value
      Dt.Points = new ArrayList(1);
      Dt.Points.add(new ArrayList(arrX.length));
      for (int i = 0; i< arrX.length; ++i)
        ((ArrayList) Dt.Points.get(0)).add(new DXFPoint(arrX[i], -arrY[i], 0));
      Xprt.addHatch(Dt);
      Dt.Point = new DXFPoint(arrX[0], -(arrY[4]-2), 0);
      Dt.FHeight = 12;
      Dt.Text = new String(strPolygon);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /**
   * Draw a Bezier.
   */
  public void sgDrawBezier() {
    int arrX[]= {70, 60,150,180,210,260,225,200,175,285,310,350,300},
        arrY[]= {80,160,180,160,140,120, 75, 90,105,105, 90, 70, 50},
        x= 240, y= 90;
    Color cl = new Color(149,122,165);
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(strBezier + " (here, only in DXF!!!)", x, y);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(strBezier);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = (Constants.convertColorRGBToDXF(cl));
      Dt.Count = arrX.length;
      Dt.Points = new ArrayList(Dt.Count);
      for (int i = 0; i< Dt.Count; ++i)
        Dt.Points.add(new DXFPoint(arrX[i], -arrY[i], 0));
      Xprt.addBezier(Dt);
      Dt.Point = new DXFPoint(x, -y, 0);
      Dt.FHeight = 12;
      Dt.Text = new String(strBezier);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
  /*
   * Draw a 3DFace.
   */
  public void sgDraw3DFace() {
    Color cl = Color.orange;
    int arrX[]= {230, 220, 250, 280, 270},
        arrY[]= {195, 165, 145, 165, 195};
    if (grphCanvas != null) {
      grphCanvas.setColor(cl);
      grphCanvas.drawString(str3DFace, arrX[0], arrY[4]-2);
      grphCanvas.drawLine(arrX[0], arrY[0], arrX[1], arrY[1]);
      grphCanvas.drawLine(arrX[1], arrY[1], arrX[2], arrY[2]);
      grphCanvas.drawLine(arrX[2], arrY[2], arrX[3], arrY[3]);
      grphCanvas.drawLine(arrX[3], arrY[3], arrX[0], arrY[0]);
      grphCanvas.drawLine(arrX[3], arrY[3], arrX[4], arrY[4]);
      grphCanvas.drawLine(arrX[4], arrY[4], arrX[0], arrY[0]);
    }
    if (Xprt != null) {
      DXFLayer Layer = new DXFLayer(str3DFace);
      Xprt.setCurrentLayer(Layer);
      DXFData Dt = new DXFData();
      Dt.LayerName = Layer.getName();
      Dt.Color = 31;
      // First 3DFace
      Dt.Point = new DXFPoint(arrX[0], -arrY[0], 0);
      Dt.Point1 = new DXFPoint(arrX[1], -arrY[1], 0);
      Dt.Point2 = new DXFPoint(arrX[2], -arrY[2], 0);
      Dt.Point3 = new DXFPoint(arrX[3], -arrY[3], 0);
      Xprt.add3DFace(Dt);
      // Second 3DFace
      Dt.Point = new DXFPoint(arrX[0], -arrY[0], 0);
      Dt.Point1 = new DXFPoint(arrX[3], -arrY[3], 0);
      Dt.Point2 = new DXFPoint(arrX[4], -arrY[4], 0);
      Dt.Point3 = new DXFPoint(arrX[0], -arrY[0], 0);
      Xprt.add3DFace(Dt);
      Dt.Point = new DXFPoint(arrX[0], -(arrY[4]-2), 0);
      Dt.FHeight = 12;
      Dt.Text = new String(str3DFace);
      Xprt.addText(Dt);
      Dt = null;
    }
  }
}
